/*
The MIT License (MIT)
 
 Copyright (c) 2015 Northern Lights Electronic Design, LLC
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 
 Original Author: Jeffrey Nygaard
 Company: Northern Lights Electronic Design, LLC
 Date: July 11, 2015
 Author Contact: JNygaard@NLEDshop.com
 Support Contact: Support@NLEDshop.com
 Written in Processing v2.0.3
 
 www.NLEDshop.com
 www.NorthernLightsElectronicDesign.com
 www.processing.org
 
 This software is offered for free and is open-source. You may use it for your commercial projects as described in the license. You may modify
 it to suit you, and share your code if you wish, as long as you give attribution and release it under the same license. It is encouraged to submit 
 improvements and bug fixes so they can be added to the main release version.
 
/*===========================================================*\
 Created for use with compatible Northern Lights Electronic Design LED Controllers
 
 Used G4P and GUI builder to create the GUI
 http://www.lagers.org.uk/g4p/
 http://www.lagers.org.uk/g4p/ref/index.html
 
 SEE README.TXT FOR USAGE INSTRUCTIONS
 
 Notes:
 - COM port names are trimmed to last 8 characters of its list() name, for Mac and Linux usage
 
 - Mini 9600 baud
 - 4 Chan DMX at 19200
 - Octo at 19200
 */

import g4p_controls.*;
import processing.serial.*;

final color cBackgroundColor = color(200, 200, 200); //background of the software

final String[] cRevLetters = {
  "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k"
};

final String cControllerID104 = "NLED 16-Bit RGBW Lamp Controller";
final String cControllerMini = "Pixel Controller Mini";
final String cControllerMiniPlus = "Pixel Controller Mini Plus";
final String cControllerID148 = "Pixel Controller Ion";
final String cControllerID43 = "NLED 4 Channel DMX Wash Controller";
final String cControllerID30 = "Quasar / 30 Channel High Current Controller";
final String cControllerID108 = "NLED OctoSequencer Controller";

int inByte = 0;    // Incoming serial data
int previnByte = 0;
int ReceiveCounter = 0;

int HardwareID = 0;
int HardwareVersion = 0;
int FirmwareVersion = 0;
int FirmwareRevision = 0;

int RequestCmdTimeOut = 0;

int cmdByte = 0;
int[] cmdData = new int[4];  

int[] DataArray = new int[8];
int tempFlags = 0;
int temp2Flags = 0;

int DeviceBaudRate = 19200; //mini uses 9600
int OpenedBaudRate = 0;

boolean RecievedOpenAck = false;
boolean SentCmdRequest = false;
boolean SentOpenRequest = false;
boolean WaitingForConfigAck = false;

String[] COMPortNames = new String[1];

Serial myPort;      // The serial port
DropDown COMPortDD;

//=====================================================================================================

void setup() 
{
  size(400, 420);
  textFont(loadFont("SansSerif.bold-48.vlw"), 48); //font saved in data folder

  //Left disabled in case of any cross platform issues
  //ImageIcon titlebaricon = new ImageIcon(loadBytes("favicon.gif"));
  //frame.setIconImage(titlebaricon.getImage());
  //frame.setResizable(false); //this work?

  createGUI();
  customGUI();

  WindowMini.setVisible(false);  
  WindowIon.setVisible(false);  
  WindowOcto.setVisible(false);  
  Window4ChanDMX.setVisible(false); 
  Window16BitLamp.setVisible(false); 
  Window30Chan.setVisible(false); 
  WindowMiniPlus.setVisible(false); 

  WindowMini.setOnTop(false);  
  WindowIon.setOnTop(false);  
  WindowOcto.setOnTop(false);  
  Window4ChanDMX.setOnTop(false); 
  Window16BitLamp.setOnTop(false); 
  Window30Chan.setOnTop(false); 
  WindowMiniPlus.setOnTop(false); 

  WindowMini.setResizable(false);  
  WindowIon.setResizable(false);  
  WindowOcto.setResizable(false);  
  Window4ChanDMX.setResizable(false); 
  Window16BitLamp.setResizable(false); 
  Window30Chan.setResizable(false); 
  WindowMiniPlus.setResizable(false); 

  //trims the COM port names to shorter length
  COMPortNames = new String[Serial.list().length];
  int varStart = 0;
  int varEnd = 0;
  for (int i = 0; i != Serial.list ().length; i++)
  {
    if (Serial.list()[i].length() > 8) { 
      varStart = Serial.list()[i].length() - 8; 
      varEnd =  Serial.list()[i].length();
    } else { 
      varStart = 0; 
      varEnd = Serial.list()[i].length();
    }
    COMPortNames[i] = Serial.list()[i].substring(varStart, varEnd);
  }

  println(Serial.list());
  println("Converted to:");
  println(COMPortNames);
  //custom drop down for COM Ports
  COMPortDD = new DropDown(COMPortNames, COMPortNames.length, 0, 20, 80, 100, 20, color(220), color(220, 220, 255), color(0), false);
} //end setup()

//=====================================================================================================

void draw() {
  background(cBackgroundColor);
  COMPortDD.display();
}

//=====================================================================================================

public void customGUI() {
}

//=====================================================================================================

void mousePressed()
{
  //if(mouseButton == RIGHT) myPort.write(1);

  if (COMPortDD.over())// && FrameOptionsDD.Selected == false)
  {
    if (COMPortDD.Selected  == true) COMPortDD.Selected = false;
    else COMPortDD.Selected = true;  
    println("Pressed DropDown Open");
  } else if (COMPortDD.overOpen() != 0 && COMPortDD.Selected == true)
  {
    println("Pressed DropDown "+COMPortDD.overOpen() );
    COMPortDD.Selected = false;
    COMPortDD.SelStr = (COMPortDD.overOpen()-1);
    CloseCOMPort();
  } //end else
  else COMPortDD.Selected = false;
} //end mousePressed()

//========================== DROP DOWN OBJECT FOR COM PORT SELECTION ============

class DropDown
{
  //use one over() for detecting the button click and if it is open, then use a different over() with the full tab page size
  String[] Labels;
  int numStrs;
  int SelStr;
  int xpos; // rect xposition  
  int ypos ; // rect yposition
  int bWidth;
  int bHeight;
  color ColBG;
  color ColHL;
  color ColOut;  
  boolean Selected;

  DropDown(String[] iLabels, int inumStrs, int iSelStr, int ixpos, int iypos, int ibWidth, int ibHeight, color iColBG, color iColHL, color iColOut, boolean iSelected)//, boolean iOpened)
  {
    Labels = iLabels; 
    numStrs = inumStrs;
    SelStr = iSelStr;
    xpos = ixpos;
    ypos = iypos;  
    bWidth = ibWidth;
    bHeight = ibHeight;
    ColBG = iColBG;
    ColHL = iColHL;
    ColOut = iColOut;
    Selected = iSelected;
    //Opened = iOpened;
  }

  void displayNormal()
  {
    strokeWeight(1);
    stroke(ColOut);

    if (Selected==true) fill(ColHL);
    else fill(ColBG);

    rect(xpos, ypos, bWidth, bHeight);  
    //adjust stroke to on higlight and invert text color
    fill(ColOut);
    textSize((bHeight/1.75));
    text(Labels[SelStr], xpos+(bWidth/20), ypos+((bHeight-(textAscent()+textDescent()))/1.5), bWidth, bHeight);

    fill(255);
    int tempSize = (bWidth/4);
    if (tempSize >  (bHeight/4)) tempSize = (bHeight);
    triangle(xpos+(bWidth-tempSize), ypos+(bHeight/4), xpos+(bWidth-tempSize)+(bHeight/2), ypos+(bHeight/4), xpos+(bWidth-tempSize)+(bHeight/4), ypos+(bHeight/4)+(bHeight/2));
    noStroke();
  }

  void display()
  {

    if (Selected == false)
    {
      displayNormal();
    } else
    {
      fill(ColBG);
      displayNormal();
      strokeWeight(1);
      stroke(ColOut);  

      for (int i = 0; i != numStrs; i++)
      {
        fill(ColBG);
        rect(5+xpos, ypos+(bHeight*(i+1)), bWidth-5, bHeight);
        fill(ColOut);
        textSize((bHeight/1.75));
        text(Labels[i], 5+xpos+((bWidth - textWidth(Labels[i]))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5)+(bHeight*(i+1)), bWidth, bHeight);
      }   //end for 
      noStroke();
    }
  }

  void displayOpen()
  {
    fill(ColBG);
    display();
    strokeWeight(1);
    stroke(ColOut);  

    for (int i = 0; i != numStrs; i++)
    {
      fill(ColBG);
      rect(5+xpos, ypos+(bHeight*(i+1)), bWidth-5, bHeight);
      fill(ColOut);
      textSize((bHeight/1.75));
      text(Labels[i], 5+xpos+((bWidth - textWidth(Labels[i]))/2), ypos+((bHeight-(textAscent()+textDescent()))/1.5)+(bHeight*(i+1)), bWidth, bHeight);
    }   //end for 
    noStroke();
  }

  boolean over() ///over the icon to open
  {
    if ( mouseX >= xpos && mouseX <= xpos+bWidth && mouseY >= ypos && mouseY <= ypos+bHeight) 
    {      
      return true;
    } else {
      return false;
    }
  }

  int overOpen()
  {
    for (int i = 0; i != numStrs; i++)
    {
      if (mouseX >= xpos && mouseX <= xpos+bWidth && mouseY >= ypos+(bHeight*(i+1)) && mouseY <= ypos+bHeight+(bHeight*(i+1)))
      {    
        return (i+1);
      }
    }
    return 0;
  }
}

